# Copyright (c) 2013, University of Bristol and University of Southampton.

## Template dependencies:
# 1LevelMod.py
# 1LevelMVMixedResponsecc.py
# 1LevelMVNormal.py
# 2LevelMod.py
# 2LevelMVNormal.py
# 2LevelRS.py
# CompleteCases.py
# Generate.py
# Merge.py
# Resp2LevelMVMixedResponsemvu.py
# Take.py

from EStat.Templating import Template

class TemplateTabulate(Template):
    'A (super)template for 2-level data that allows for missing values in explanatory or response variables, and can handle normal or categorical variables. After the user separately specifies imputation and model of interest (MOI), multiply-imputed datasets are produced, and the MOI fitted to each (via a variety of other templates); these are combined using Rubin\'s rules, and the results for the final MOI fit are then returned (together with results from a complete case analysis, for comparison). If your computer has multiple processors, these will be used in parallel for the imputation models.'

    __version__ = '1.0.0'

    tags = [ 'Multiple imputation', 'Missing data', '1-Level', '2-Level', 'Model', 'Multivariate response', 'Normal', 'Ordered multinomial', 'Random slopes', 'Unordered multinomial' ]
    engines = ['Python_script']

    inputs = '''
numlevs = Text('Do either the model of interest (MOI) and/or imputation have two levels?', ['Yes', 'No, they are both 1-level'])

###specify MOI:
if numlevs == 'No, they are both 1-level':
    MOIdist = Text('The next few questions will ask you to specify your MOI. What distribution would you like to use to model the response variable(s)?', ['Normal', 'Binary', 'Poisson', 'Multivariate Normal'])
    if MOIdist == 'Normal' or MOIdist == 'Binary' or MOIdist == 'Poisson':
        templatename = Text(value = '1LevelMod')
    else:
        templatename = Text(value = '1LevelMVNormal')

if numlevs == 'Yes':
    L2ID = IDVector('Level 2 ID: ')
    if L2ID !="":
        MOIlevel = Text('The next few questions will ask you to specify your MOI. In your MOI, do you want to model 1 or 2 levels?', ['1 level', '2 levels'])
        MOIdist = Text('What distribution would you like to use to model the response variable(s) in your MOI?', ['Normal', 'Binary', 'Poisson', 'Multivariate Normal'])
        if MOIdist == 'Normal' or MOIdist == 'Binary' or MOIdist == 'Poisson':
            if MOIlevel == '2 levels':
                MOIslope = Boolean('In your MOI, do you want to fit random slope/coefficient(s)?', help="If you would like to fit a random slope/coefficicent(s) model, make sure that you include the response(s) in your MOI, AND the variables whose coefficients you wish to randomly-vary at level 2, as responses in the imputation model.")
                if MOIslope:
                    templatename = Text(value = '2LevelRS')
                else:
                    templatename = Text(value = '2LevelMod')
            else:
                    templatename = Text(value = '1LevelMod')                
        else:
            if MOIlevel == '2 levels':
                templatename = Text(value = '2LevelMVNormal')
            else:
                templatename = Text(value = '1LevelMVNormal')

if templatename == '2LevelMVNormal' or templatename == '1LevelMVNormal':
    #Not used, but D referred to in pythonscript, below:
    D = Text(value = 'MVNormal')
    y = DataMatrix('In your MOI, what are the response variables? ')
    for i in range(0, len(y)):
        context['x'+str(i+1)] = DataMatrix('Explanatory variables for response ' + y[i] + ' in your MOI: ', allow_cat = True)
    if len(y) > 1:
        priors = Text('Level 1 priors: ', ['Uniform', 'Wishart'])
        if priors == 'Wishart':
            R = List('R matrix: ')
            v = Integer('Degrees of Freedom:')
        if numlevs == 'Yes':
            priors2 = Text('Level 2 priors: ', ['Uniform', 'Wishart'])
            if priors2 == 'Wishart':
                R2 = List('R matrix: ')
                v2 = Integer('Degrees of Freedom:')
    mv = Boolean('Use MVNormal update for beta?: ')
else:
    if MOIdist == 'Normal':
        D = Text(value = 'Normal')
    elif MOIdist == 'Binary':
        D = Text(value = 'Binomial')
    elif MOIdist == 'Poisson':
        D = Text(value = 'Poisson')
    y = DataVector('In your MOI, what is the response variable? ')
    if D == 'Binomial':
        link = Text('In your MOI, what link function would you like to use? ', ['logit', 'probit', 'cloglog'])
    if D == 'Poisson':
        link = Text(value = 'ln')
        offset = Boolean('Is there an offset? ')
        if offset:
            n = DataVector('offset: ')
    x = DataMatrix('Explanatory variables for MOI: ', allow_cat = True)
if templatename == '2LevelRS':
    x2 = DataMatrix('In your MOI, which explanatory variables are random at level 2? ', allow_cat = True)
    priors = Text('Priors: ', ['Uniform', 'Wishart'])
    if priors == 'Wishart':
        R = List('R matrix: ')
        v = Integer('Degrees of Freedom:')

###specify imputation model:
###(changed so asks at a more logical pace in eBook)
if templatename == '2LevelMVNormal' or templatename == '1LevelMVNormal':
    lenx = len(x1)
else:
    lenx = len(x)
if lenx > 0:
    yimp1 = DataMatrix('The next few questions will now ask about your IMPUTATION MODEL. What responses are at level 1? ')
    for i in range(0, len(yimp1)):
        context['bin1_'+str(i+1)] = Text('Response Type for response ' + yimp1[i] + ': ', ['Normal', 'Binary', 'Ordered', 'Unordered'])
        
    for i in range(0, len(yimp1)):
        if(context['bin1_'+str(i+1)] == 'Ordered'):
            context['ncats'+yimp1[i]] = Integer('Number of categories for ' + yimp1[i] + ': ', help="Your categories, as represented in your dataset, need to be numbered from zero, sequentially in steps of one (i.e. 0,1,2 if you had 3 categories).")
            L1UserOrderedCats = numpy.unique(GUIObject.interface.data[yimp1.data].variables[yimp1[i]]['data'].compressed())
            L1OrderedCatChecker = numpy.array(range(0, context['ncats'+yimp1[i]]))
            if not numpy.all(L1UserOrderedCats == L1OrderedCatChecker):
                context['msg' + str(i)] = Feedback('ERROR: the variable you have specified as ordered (' + str(yimp1[i]) + ') needs to be numbered from zero, sequentially in steps of one (i.e. 0,1,2 if you had 3 categories); otherwise the model will not fit. If you wish to fit this variable as ordered, you therefore need to transform it appropriately in your dataset.')
        if(context['bin1_'+str(i+1)] == 'Unordered'):
            context['ncats'+yimp1[i]] = Integer('Number of categories for ' + yimp1[i] + ': ', help="Your categories, as represented in your dataset, need to be numbered from zero, sequentially in steps of one (i.e. 0,1,2 if you had 3 categories).")
            L1UserUnorderedCats = numpy.unique(GUIObject.interface.data[yimp1.data].variables[yimp1[i]]['data'].compressed())
            L1UnorderedCatChecker = numpy.array(range(0, context['ncats'+yimp1[i]]))
            if not numpy.all(L1UserUnorderedCats == L1UnorderedCatChecker):
                context['msg' + str(i)] = Feedback('ERROR: the variable you have specified as unordered (' + str(yimp1[i]) + ') needs to be numbered from zero, sequentially in steps of one (i.e. 0,1,2 if you had 3 categories); otherwise the model will not fit. If you wish to fit this variable as unordered, you therefore need to transform it appropriately in your dataset.')
        if(context['bin1_'+str(i+1)] == 'Unordered'):
            for j in range(0, context['ncats'+yimp1[i]]-1):
                context['ximp1' + '_' + str(i) + '_' + str(j)] = DataMatrix('Explanatory variables for category ' + str(j) + ' in imputation model: ', allow_cat = True)
        else:
            context['ximp1' + '_' + str(i)] = DataMatrix('Explanatory variables for ' + yimp1[i] + ' in imputation model: ', allow_cat = True)

if numlevs == 'Yes':
    want2 = Boolean('Are there any responses at level 2 for imputation model?: ')
    if want2:
        yimp2 = DataMatrix('Responses at level 2 for imputation model: ')
        for i in range(0, len(yimp2)):
            context['bin2_'+str(i+1)] = Text('Response Type for response ' + yimp2[i] + ': ', ['Normal', 'Binary', 'Ordered', 'Unordered'])
        for i in range(0, len(yimp2)):
            if(context['bin2_'+str(i+1)] == 'Ordered'):
                context['ncats'+yimp2[i]] = Integer('Number of categories for ' + yimp2[i] + ': ', help="Your categories, as represented in your dataset, need to be numbered from zero, sequentially in steps of one (i.e. 0,1,2 if you had 3 categories).")
                L2UserOrderedCats = numpy.unique(GUIObject.interface.data[yimp2.data].variables[yimp2[i]]['data'].compressed())
                L2OrderedCatChecker = numpy.array(range(0, context['ncats'+yimp2[i]]))
                if not numpy.all(L2UserOrderedCats == L2OrderedCatChecker):
                    context['msg' + str(i)] = Feedback('ERROR: the variable you have specified as ordered (' + str(yimp2[i]) + ') needs to be numbered from zero, sequentially in steps of one (i.e. 0,1,2 if you had 3 categories); otherwise the model will not fit. If you wish to fit this variable as ordered, you therefore need to transform it appropriately in your dataset.')
            if(context['bin2_'+str(i+1)] == 'Unordered'):
                context['ncats'+yimp2[i]] = Integer('Number of categories for ' + yimp2[i] + ': ', help="Your categories, as represented in your dataset, need to be numbered from zero, sequentially in steps of one (i.e. 0,1,2 if you had 3 categories).")
                L2UserUnorderedCats = numpy.unique(GUIObject.interface.data[yimp2.data].variables[yimp2[i]]['data'].compressed())
                L2UnorderedCatChecker = numpy.array(range(0, context['ncats'+yimp2[i]]))
                if not numpy.all(L2UserUnorderedCats == L2UnorderedCatChecker):
                    context['msg' + str(i)] = Feedback('ERROR: the variable you have specified as unordered (' + str(yimp2[i]) + ') needs to be numbered from zero, sequentially in steps of one (i.e. 0,1,2 if you had 3 categories); otherwise the model will not fit. If you wish to fit this variable as unordered, you therefore need to transform it appropriately in your dataset.')
            if(context['bin2_'+str(i+1)] == 'Unordered'):
                for j in range(0, context['ncats'+yimp2[i]]-1):
                    context['ximp2' + '_' + str(i) + '_' + str(j)] = DataMatrix('Explanatory variables for category ' + str(j) + ' at level 2 for imputation model: ', allow_cat = True)
            else:
                context['ximp2' + '_' + str(i)] = DataMatrix('Explanatory variables for response ' + yimp2[i] + ' at level 2 for imputation model: ', allow_cat = True)

        # Check response is really at level 2
        l2var = GUIObject.interface.data[L2ID.data].variables[L2ID]['data']
        (l2vals, l2index, l2inverse) = numpy.unique(l2var, return_index=True, return_inverse=True)
        for i in range(0, len(yimp2)):
            datavar = GUIObject.interface.data[yimp2.data].variables[yimp2[i]]['data']
            # Note: could be slow as we check all the data for each l2 value
            #for j in range(0, len(l2vals)):
            #    if not datavar[l2var == l2vals[j]].var() == 0:
            #        context['msg' + str(i)] = Feedback(str(yimp2[i]) + ' is not a level 2 variable')
            # So use the following instead:
            if not numpy.all(datavar == datavar[l2index][l2inverse]):
                context['msg' + str(i)] = Feedback('ERROR: ' + str(yimp2[i]) + ' is not a level 2 variable. Please remove and select an appropriate level 2 variable.')
            if(context['bin2_'+str(i+1)] == 'Unordered'):
                for j in range(0, len(context['ximp2' + '_' + str(i) + '_' + str(j)])):
                    for k in range(0, context['ncats'+yimp2[i]]-1):
                        datavar = GUIObject.interface.data[context['ximp2' + '_' + str(i) + '_' + str(k)].data].variables[context['ximp2' + '_' + str(i) + '_' + str(k)][i]]['data']
                        if not numpy.all(datavar == datavar[l2index][l2inverse]):
                            context['msg' + str(i) + '_' + str(j) + '_' + str(k)] = Feedback(str(context['ximp2' + '_' + str(i) + '_' + str(k)][i]) + ' is not a level 2 variable')
            else:
                for j in range(0, len(context['ximp2' + '_' + str(i)])):
                    datavar = GUIObject.interface.data[context['ximp2' + '_' + str(i)].data].variables[context['ximp2' + '_' + str(i)][j]]['data']
                    if not numpy.all(datavar == datavar[l2index][l2inverse]):
                        context['msg' + str(i) + '_' + str(j)] = Feedback(str(context['ximp2' + '_' + str(i)][j]) + ' is not a level 2 variable')

###changed condmarg from Boolean to TextRadio so doesn't have default in eBook (led to problems of it being selected in error, by default, before user explicitly asked for input)
#condmarg = Boolean('Use conditional marginal algorithm: ')
condmarg = TextRadio('Use conditional marginal algorithm?', ['Yes', 'No'])

numimpute = Integer('Number of imputed data sets: ')
imputeevery = Integer('Number of iterations between imputations: ', help="NB: burnin for imputation model fixed at 1000. If parallel chains are run, the 1st iteration after burnin is used; if not parallel chains (i.e. if fewer processors than imputed datasets requested), then this input question pertains to the interval at which iterations sampled.")

burnin=Integer('Length of burnin for MOI: ')
iterations=Integer('Number of iterations for MOI: ')
'''

    pythonscript = '''
import numpy

import multiprocessing

import EStat
from EStat.Templating import *

#import sys
#sys.path.append(os.path.abspath(os.path.join('..', '..', 'apps', 'cmdtest')))

from cmdtest import RunStatJR
from EStat.DTAFile import DTAFile
from EStat.stats.utils.MCMCGraph import MCMCGraph
import datasets

print 'Datasets: ' + str(datasets.datasets.keys())

if D == 'Binomial':
    datafile.addvariable("autodenom",data=numpy.ones(datafile.nobs))

datasets.datasets['workdata'] = datafile

nchain = multiprocessing.cpu_count()
nperchain = int(numimpute) / nchain
nextraimpute = int(numimpute) % nchain

imputes = []
for i in range(0, nchain):
    iters = []
    for j in range(0, nperchain):
        iters.append(j * int(imputeevery))
    if i < nextraimpute:
        iters.append(nperchain * int(imputeevery))
    if len(iters) > 0:
        imputes.append(iters)

moivars = set()
if templatename == '2LevelMVNormal' or templatename == '1LevelMVNormal':
    moivars.update(list(y))
    for i in range(0, len(y)):
        moivars.update(list(context['x'+str(i+1)].orignames))
else:
    moivars.update([str(y)])
    moivars.update(list(x.orignames))
    if templatename == '2LevelRS':
        moivars.update(list(x2.orignames))
    if D == 'Binomial':
        moivars.update(["autodenom"])
    if D == 'Poisson':
        moivars.update([str(n)])

if numlevs == 'Yes':
    moivars.update([str(L2ID)])

print moivars
m = RunStatJR(template='CompleteCases', dataset='workdata', invars = {'vars': ','.join(moivars), 'outdata': 'CompleteData'}, estoptions = {'Engine': 'Python_script'})

if numlevs == 'Yes' and want2:
    l2vars = list(yimp2)
    for i in range(0, len(yimp2)):
        if context['bin2_'+str(i+1)] == 'Unordered':
            for j in range(0, context['ncats'+yimp2[i]]-1):
                l2vars += list(context['ximp2' + '_' + str(i) + '_' + str(j)])
        else:
            l2vars += list(context['ximp2' + '_' + str(i)])
        if context['bin2_'+str(i+1)] == 'Binomial':
            l2vars += list("autodenom")
        if context['bin2_'+str(i+1)] == 'Poisson':
            l2vars += list(str(n))

if numlevs == 'Yes' and want2:
    m = RunStatJR(template='Take', dataset='workdata', invars = {'LID': str(L2ID), 'incols': ','.join(l2vars), 'outdata': '_L2Data'}, estoptions = {'Engine': 'Python_script'})

print 'Datasets: ' + str(datasets.datasets.keys())

inputs = {}
    
if numlevs == 'No, they are both 1-level':
    inputs['y'] = ','.join(yimp1)
if numlevs == 'Yes':
    inputs['y1'] = ','.join(yimp1)
    if want2:
        inputs['y2'] = ','.join(yimp2)
    inputs['L2Data'] = '_L2Data'
    inputs['L2ID'] = str(L2ID)
    inputs['storeresid'] = 'No'

inputs['outdata'] = 'out'

inputs['condmarg'] = str(condmarg)

def inputlist(obj):
    varlist = []
    for i in range(0, len(obj.orignames)):
        if obj.orignames[i] in obj.categorical:
            varlist.append(obj.orignames[i] + ':cat') 
        else:
            varlist.append(obj.orignames[i])
    return varlist

if numlevs == 'No, they are both 1-level':
    for i in range(0, len(yimp1)):
        if(context['bin1_'+str(i+1)] == 'Binary'):
            context['bin1_'+str(i+1)] = Text(value = 'Binomial')
        inputs['bin' + str(i + 1)] = str(context['bin1_'+str(i+1)])
        if(context['bin1_'+str(i+1)] == 'Ordered'):
            inputs['ncats'+yimp1[i]] = str(context['ncats'+yimp1[i]])
        if(context['bin1_'+str(i+1)] == 'Unordered'):
            inputs['ncats'+yimp1[i]] = str(context['ncats'+yimp1[i]])
            for j in range(0, context['ncats'+yimp1[i]] - 1):
                inputs['x'+str(i+1)+'_'+str(j)] = ','.join(inputlist(context['ximp1' + '_' + str(i) + '_' + str(j)])) #'_cons'
        else:
            inputs['x'+str(i+1)] = ','.join(inputlist(context['ximp1' + '_' + str(i)])) #'_cons'

if numlevs == 'Yes':
    for i in range(0, len(yimp1)):
        if(context['bin1_'+str(i+1)] == 'Binary'):
            context['bin1_'+str(i+1)] = Text(value = 'Binomial')
        inputs['bin1_' + str(i + 1)] = str(context['bin1_'+str(i+1)])
        if(context['bin1_'+str(i+1)] == 'Ordered'):
            inputs['ncats'+yimp1[i]] = str(context['ncats'+yimp1[i]])
        if(context['bin1_'+str(i+1)] == 'Unordered'):
            inputs['ncats'+yimp1[i]] = str(context['ncats'+yimp1[i]])
            for j in range(0, context['ncats'+yimp1[i]] - 1):
                inputs['x1_'+str(i+1)+'_'+str(j)] = ','.join(inputlist(context['ximp1' + '_' + str(i) + '_' + str(j)])) #'_cons'
        else:
            inputs['x1_'+str(i+1)] = ','.join(inputlist(context['ximp1' + '_' + str(i)])) #'_cons'

    if want2:
        for i in range(0, len(yimp2)):
            if(context['bin2_'+str(i+1)] == 'Binary'):
                context['bin2_'+str(i+1)] = Text(value = 'Binomial')
            inputs['bin2_' + str(i + 1)] = str(context['bin2_'+str(i+1)])
            if(context['bin2_'+str(i+1)] == 'Ordered'):
                inputs['ncats'+yimp2[i]] = str(context['ncats'+yimp2[i]])
            if(context['bin2_'+str(i+1)] == 'Unordered'):
                inputs['ncats'+yimp2[i]] = str(context['ncats'+yimp2[i]])
                for j in range(0, context['ncats'+yimp2[i]] - 1):
                    inputs['x2_'+str(i+1)+'_'+str(j)] = ','.join(inputlist(context['ximp2' + '_' + str(i) + '_' + str(j)])) #'_cons'
            else:
                inputs['x2_'+str(i+1)] = ','.join(inputlist(context['ximp2' + '_' + str(i)])) #'_cons'

omega_e1_diag = []

for i in range(0, len(yimp1)):
    if context['bin1_'+str(i+1)] == 'Normal':
        omega_e1_diag.append(str(datafile.variables[yimp1[i]]['data'].var()))
    elif context['bin1_'+str(i+1)] == 'Unordered':
        for j in range(0, int(context['ncats'+yimp1[i]]) - 1):
            omega_e1_diag.append('1')
    else:
        omega_e1_diag.append('1')

omega_e1_full = []
for i in range(0, len(omega_e1_diag)):
    for j in range(0, i+1):
        if i == j:
            omega_e1_full.append(omega_e1_diag[i])
        else:
            omega_e1_full.append('0')

if numlevs == 'No, they are both 1-level':
    inputs['omega_e'] = ','.join(omega_e1_full)

if numlevs == 'Yes':
    inputs['omega_e1'] = ','.join(omega_e1_full)
    inputs['anyl2resp'] = str(want2)

estinputs = {}
estinputs['Engine'] = 'CustomC'
estinputs['defaultsv'] = 'Yes'
estinputs['defaultalg'] = 'Yes'
estinputs['burnin'] = '1000'
estinputs['seed'] = '1'
estinputs['thinning'] = '1'
estinputs['makepred'] = 'No'
#estinputs['nchains'] = str(int(numimpute)) #'1'
#estinputs['iterations'] = str(int(imputeevery)) #str(int(numimpute) * int(imputeevery))
#estinputs['imputeiters'] = str(int(imputeevery)) #','.join([str(i) for i in numpy.arange(int(imputeevery), (int(numimpute) * int(imputeevery)) + 1, int(imputeevery))])
# Note: impute iterations must be the same for all chains, so we are potentially doing more iterations than necessary
estinputs['nchains'] = str(len(imputes))
estinputs['iterations'] = str(imputes[0][-1]+1)
estinputs['imputeiters'] = ','.join([str(i) for i in imputes[0]])

if numlevs == 'No, they are both 1-level':
    m = RunStatJR(template='1LevelMVMixedResponsecc', dataset='workdata', invars = inputs, estoptions = estinputs)
if numlevs == 'Yes':
    m = RunStatJR(template='Resp2LevelMVMixedResponsemvu', dataset='workdata', invars = inputs, estoptions = estinputs)

for k in m.eng.inputs.keys():
    outputs['Imputation_Model_' + k] = m.eng.inputs[k]

for k in m.eng.outputs.keys():
    outputs['Imputation_Model_' + k] = m.eng.outputs[k]

if numlevs == 'Yes' and want2:
    for i in range(0, len(imputes)):
        for j in range(0, len(imputes[i])):
            m = RunStatJR(template='Merge', dataset='impute_datafile_chain' + str(i) + '_iter' + str(imputes[i][j]), invars = {'dataset2': 'impute__L2Data_chain' + str(i) + '_iter' + str(imputes[i][j]), 'outdata': 'impute_datafile_chain' + str(i) + '_iter' + str(imputes[i][j]), 'idcol': str(L2ID)}, estoptions = {'Engine': 'Python_script'})
            for k in m.eng.outputs.keys():
                outputs['Imputation_Model_' + k] = m.eng.outputs[k]

moiinputs = {}
if numlevs == 'Yes':
    moiinputs['L2ID'] = str(L2ID)
    moiinputs['storeresid'] = 'No'
if templatename == '2LevelMVNormal' or templatename == '1LevelMVNormal':
    moiinputs['y'] = ",".join(y)
    for i in range(0, len(y)):
        moiinputs['x'+str(i+1)] = ",".join(context['x'+str(i+1)])
    if len(y) > 1:
        moiinputs['priors'] = str(priors)
        if priors == 'Wishart':
            moiinputs['R'] = ",".join(R)
            moiinputs['v'] = str(v)
        if numlevs == 'Yes':
            moiinputs['priors2'] = str(priors2)
            if priors2 == 'Wishart':
                moiinputs['R2'] = ",".join(R2)
                moiinputs['v2'] = str(v2)
    moiinputs['mv'] = str(mv)
else:
    moiinputs['y'] = str(y)
    moiinputs['x'] = ",".join(inputlist(x))
    moiinputs['D'] = str(D)
    if D == 'Binomial':
        moiinputs['n'] = "autodenom"
        moiinputs['link'] = str(link)
    if D == 'Poisson':
        moiinputs['link'] = str(link)
        moiinputs['offset'] = str(offset)
        if offset:
            moiinputs['n'] = str(n)

if templatename == '2LevelRS':
    moiinputs['x2'] = ",".join(x2)
    moiinputs['priors'] = str(priors)
    if priors == 'Wishart':
        moiinputs['R'] = ",".join(R)
        moiinputs['v'] = str(v)

seed = 0
moi = []
z = 0
for i in range(0, len(imputes)):
    for j in range(0, len(imputes[i])):
        seed += 1
        m = RunStatJR(template=str(templatename), dataset='impute_datafile_chain' + str(i) + '_iter' + str(imputes[i][j]), invars=moiinputs, estoptions={'burnin': str(burnin), 'iterations': str(iterations), 'thinning': '1', 'seed': str(seed), 'nchains': '1', 'Engine':'eStat', 'defaultalg':'Yes', 'defaultsv':'Yes', 'makepred':'No', 'outdata':'imp_'+str(i+1)+'_'+str(j)})
        for k in m.eng.inputs.keys():
            outputs['Model' + str(z + 1) + '_' + k] = m.eng.inputs[k]
        for k in m.eng.outputs.keys():
            outputs['Model' + str(z + 1) + '_' + k] = m.eng.outputs[k]
        moi.append(m)
        z += 1
combinedchains = {}
for p in moi[0].eng.outputs['Chains'].chains.keys():
    chains = moi[0].eng.outputs['Chains'].chains[p]
    combinedchains[p] = numpy.empty((len(moi), chains.size))

meanDIC = 0
meanpD = 0
for i in range(0, len(moi)):
    for p in moi[i].eng.outputs['Chains'].chains.keys():
        chains = moi[i].eng.outputs['Chains'].chains[p]
        combinedchains[p][i, :] = chains
    meanDIC += moi[i].eng.outputs['ModelResults'].content['modelstats']['DIC']
    meanpD += moi[i].eng.outputs['ModelResults'].content['modelstats']['pD']

meanDIC /= len(moi)
meanpD /= len(moi)

combinedresults = ModelOutput()
for p in combinedchains.keys():
    combinedresults.add(p, 'chains', combinedchains[p])
    outputs['Combined_' + p + '.svg'] = MCMCGraph(p, combinedchains[p], moi[0].eng.directory)
combinedresults.summarisechains()
outputs['CombinedResults'] = combinedresults

completemoi = RunStatJR(template=str(templatename), dataset='CompleteData', invars= moiinputs, estoptions={'burnin': str(burnin), 'iterations': str(iterations), 'thinning': '1', 'seed': str(i + 1), 'nchains': '1', 'Engine':'eStat', 'defaultalg':'Yes', 'defaultsv':'Yes', 'makepred':'No', 'outdata':'imp'+str(i+1)})
for k in completemoi.eng.inputs.keys():
    outputs['CompleteCasesModel' + '_' + k] = completemoi.eng.inputs[k]

for k in completemoi.eng.outputs.keys():
    outputs['CompleteCasesModel' + '_' + k] = completemoi.eng.outputs[k]

tabout = TabularOutput()
tabout.column_headings = ['parameter', 'mean', 'standard error', 'variable']

tabout.add_row('Mean DIC', [meanDIC, '', ''])
tabout.add_row('Mean pD', [meanpD, '', ''])

for p in moi[0].eng.outputs['ModelResults'].content['paramstats']['mean'].keys():
    if 'variable' in moi[0].eng.outputs['ModelResults'].content['paramstats'] and p in moi[0].eng.outputs['ModelResults'].content['paramstats']['variable']:
        varname = moi[0].eng.outputs['ModelResults'].content['paramstats']['variable'][p]
    else:
        varname = ''

    thetahat = 0
    for i in range(0, len(moi)):
        thetahat += moi[i].eng.outputs['ModelResults'].content['paramstats']['mean'][p]
    thetahat /= len(moi)

    vbar = 0
    for i in range(0, len(moi)):
        vbar += numpy.power(moi[i].eng.outputs['ModelResults'].content['paramstats']['sd'][p], 2)
    vbar /= len(moi)

    B = 0
    for i in range(0, len(moi)):
        B += numpy.power((moi[i].eng.outputs['ModelResults'].content['paramstats']['mean'][p] - thetahat), 2) / (len(moi) - 1)

    vartheta = vbar + (1 + (1/len(moi)))*B

    if not p.startswith('tau') and not p=='deviance':
        tabout.add_row(p, [thetahat, numpy.sqrt(vartheta), varname])

outputs['ResultsTable:Imputation'] = tabout

taboutcomp = TabularOutput()
taboutcomp.column_headings = ['parameter', 'mean', 'standard error', 'variable']
taboutcomp.add_row('DIC', [completemoi.eng.outputs['ModelResults'].content['modelstats']['DIC'], '', ''])
taboutcomp.add_row('pD', [completemoi.eng.outputs['ModelResults'].content['modelstats']['pD'], '', ''])
for p in completemoi.eng.outputs['ModelResults'].content['paramstats']['mean'].keys():
    if 'variable' in completemoi.eng.outputs['ModelResults'].content['paramstats'] and p in completemoi.eng.outputs['ModelResults'].content['paramstats']['variable']:
        varname = completemoi.eng.outputs['ModelResults'].content['paramstats']['variable'][p]
    else:
        varname = ''

    meancomplete = completemoi.eng.outputs['ModelResults'].content['paramstats']['mean'][p]
    varcomplete = numpy.power(completemoi.eng.outputs['ModelResults'].content['paramstats']['sd'][p], 2)
    if not p.startswith('tau') and not p=='deviance':
        taboutcomp.add_row(p, [meancomplete, numpy.sqrt(varcomplete), varname])

outputs['ResultsTable:CompleteCases'] = taboutcomp
'''
